// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <stddef.h>

#include "MathMLRowElement.hh"
#include "MathMLScriptElement.hh"
#include "RenderingEnvironment.hh"

MathMLScriptElement::MathMLScriptElement(TagId id): MathMLScriptCommonElement(id)
{
  subScript = superScript = NULL;
}

MathMLScriptElement::~MathMLScriptElement()
{
}

void
MathMLScriptElement::Normalize()
{
  unsigned n = (IsA() == TAG_MSUBSUP) ? 3 : 2;

  while (content.GetSize() > n) {
    MathMLElement* elem = content.RemoveLast();
    delete elem;
  }

  while (content.GetSize() < n) {
    MathMLRowElement* mrow = new MathMLRowElement;
    mrow->SetParent(this);
    content.Append(mrow);
  }

  // BEWARE: normalization has to be done here, since it may
  // change the content!!!
  MathMLContainerElement::Normalize();

  base = content.GetFirst();
  assert(base != NULL);

  if (IsA() == TAG_MSUB) subScript = content.GetLast();
  else if (IsA() == TAG_MSUP) superScript = content.GetLast();
  else {
    subScript   = content.Get(1);
    superScript = content.Get(2);
  }
}

void
MathMLScriptElement::Setup(RenderingEnvironment* env)
{
  assert(env != NULL);

  base->Setup(env);

  env->Push();
  env->AddScriptLevel(1);
  env->SetDisplayStyle(false);

  const Value* value = NULL;

  if (subScript != NULL) {
    subScript->Setup(env);

    value = GetAttributeValue(ATTR_SUBSCRIPTSHIFT, env, false);
    if (value != NULL) {
      assert(value->IsNumberUnit());

      UnitValue unitValue = value->ToNumberUnit();
      assert(!unitValue.IsPercentage());

      subMinShift = env->ToScaledPoints(unitValue);
      delete value;
    }
  }

  if (superScript != NULL) {
    superScript->Setup(env);

    value = GetAttributeValue(ATTR_SUPERSCRIPTSHIFT, env, false);
    if (value != NULL) {
      assert(value->IsNumberUnit());
      
      UnitValue unitValue = value->ToNumberUnit();
      assert(!unitValue.IsPercentage());

      superMinShift = env->ToScaledPoints(unitValue);
      delete value;
    }
  }

  MathMLScriptCommonElement::Setup(env);

  env->Drop();
}

void
MathMLScriptElement::DoBoxedLayout(LayoutId id, BreakId bid, scaled maxWidth)
{
  assert(base != NULL);

  base->DoBoxedLayout(id, BREAK_NO, maxWidth / 2);
  if (subScript != NULL) subScript->DoBoxedLayout(id, BREAK_NO, maxWidth / 2);
  if (superScript != NULL) superScript->DoBoxedLayout(id, BREAK_NO, maxWidth / 2);

  const BoundingBox& baseBox = base->GetBoundingBox();
  BoundingBox subScriptBox;
  BoundingBox superScriptBox;

  subScriptBox.Null();
  if (subScript != NULL) subScriptBox = subScript->GetBoundingBox();

  superScriptBox.Null();
  if (superScript != NULL) superScriptBox = superScript->GetBoundingBox();

  DoLayoutAux(baseBox, subScriptBox, superScriptBox);

  box = baseBox;
  box.width += scriptSpacing + scaledMax(subScriptBox.width, superScriptBox.width);

  if (subScript != NULL) {
    box.ascent  = scaledMax(box.ascent, subScriptBox.ascent - subShift);
    box.descent = scaledMax(box.descent, subScriptBox.descent + subShift);
  }

  if (superScript != NULL) {
    box.ascent  = scaledMax(box.ascent, superScriptBox.ascent + superShift);
    box.descent = scaledMax(box.descent, superScriptBox.descent - superShift);
  }

  ConfirmLayout(id);
}

void
MathMLScriptElement::SetPosition(scaled x, scaled y)
{
  assert(base != NULL);

  position.x = x;
  position.y = y;

  scaled scriptXOffset = base->GetBoundingBox().width + scriptSpacing;
  base->SetPosition(x, y);
  if (subScript != NULL) subScript->SetPosition(x + scriptXOffset, y + subShift);
  if (superScript != NULL) superScript->SetPosition(x + scriptXOffset, y - superShift);
}
