/*
 * Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
 * 
 * This file is part of GtkMathView, a Gtk widget for MathML.
 * 
 * GtkMathView is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * GtkMathView is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GtkMathView; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 * For details, see the GtkMathView World-Wide-Web page,
 * http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
 * <luca.padovani@cs.unibo.it>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "gtkmathview.h"
#include "guiGTK.h"

static GtkWidget* window;
static GtkWidget* main_area;
static GtkWidget* scrolled_area;

static void create_widget_set(void);
static GtkWidget* get_main_menu(void);
static void file_open(GtkWidget*, gpointer);
static void options_font_size(GtkWidget*, guint);
static void options_link_color(GtkWidget*, gpointer);
static void options_verbose(GtkWidget*, gpointer);
static void options_select_mode(GtkWidget*, gpointer);
static void help_about(GtkWidget*, gpointer);

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",         NULL,         NULL,          0, "<Branch>" },
  { "/File/_Open...", "<control>O", file_open,     0, NULL },
  { "/File/sep1",     NULL,         NULL,          0, "<Separator>" },
  { "/File/_Quit",    "<control>Q", gtk_main_quit, 0, NULL },

  { "/_Options",      NULL,         NULL,          0, "<Branch>" },
  { "/Options/Default _Font Size", NULL, NULL,     0, "<Branch>" },
  { "/Options/Default Font Size/8pt",  NULL, options_font_size,  8, NULL },
  { "/Options/Default Font Size/10pt", NULL, options_font_size, 10, NULL },
  { "/Options/Default Font Size/12pt", NULL, options_font_size, 12, NULL },
  { "/Options/Default Font Size/14pt", NULL, options_font_size, 14, NULL },
  { "/Options/Default Font Size/18pt", NULL, options_font_size, 18, NULL },
  { "/Options/Default Font Size/24pt", NULL, options_font_size, 24, NULL },
  { "/Options/sep1",  NULL,         NULL,          0, "<Separator>" },
  { "/Options/Link _Color...", NULL,     options_link_color, 0, NULL },
  { "/Options/sep2",  NULL,         NULL,          0, "<Separator>" },
  { "/Options/_Verbose", NULL,      options_verbose,          0, "<ToggleItem>" },
  { "/Options/_Select Presentation", NULL, options_select_mode, 0, "<ToggleItem>" },

  { "/_Help" ,        NULL,         NULL,          0, "<LastBranch>" },
  { "/Help/About...", NULL,         help_about,    0, NULL }
};

void
GUI_init(int *argc, char ***argv, char *title, unsigned width, unsigned height)
{
  gtk_init(argc, argv);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), title);
  gtk_window_set_default_size(GTK_WINDOW(window), width, height);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event", (GtkSignalFunc) gtk_main_quit, NULL);

  create_widget_set();

  gtk_widget_show(window);
}

void
GUI_uninit()
{
}

int
GUI_load_document(const char* name)
{
  GtkMathView* math_view;

  g_return_val_if_fail(main_area != NULL, -1);
  g_return_val_if_fail(GTK_IS_MATH_VIEW(main_area), -1);

  math_view = GTK_MATH_VIEW(main_area);

  if (!gtk_math_view_load(math_view, name)) return -1;
    
  return 0;
}

void
GUI_unload_document()
{
  GtkMathView* math_view;

  g_return_if_fail(main_area != NULL);
  g_return_if_fail(GTK_IS_MATH_VIEW(main_area));

  math_view = GTK_MATH_VIEW(main_area);

  gtk_math_view_unload(math_view);
}

void
GUI_dump_document()
{
  GtkMathView* math_view;

  g_return_if_fail(main_area != NULL);
  g_return_if_fail(GTK_IS_MATH_VIEW(main_area));

  math_view = GTK_MATH_VIEW(main_area);

  gtk_math_view_dump(math_view);
}

void
GUI_run()
{
  gtk_main();
}

static void
store_filename(GtkFileSelection *selector, GtkWidget* user_data)
{
  GtkMathView* math_view;
  gchar* selected_filename;

  selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data));

  math_view = GTK_MATH_VIEW(main_area);

  gtk_math_view_load(math_view, selected_filename);
}

static void
file_open(GtkWidget* widget, gpointer data)
{
  GtkWidget* fs = gtk_file_selection_new("Open File");

  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(fs)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (store_filename), (gpointer) fs);
                             
  /* Ensure that the dialog box is destroyed when the user clicks a button. */
     
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fs)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) fs);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fs)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) fs);
     
  /* Display that dialog */
     
  gtk_widget_show (fs);
}

static void
options_font_size(GtkWidget* widget, guint size)
{
  GtkMathView* math_view;
  SelectMode mode;

  g_return_if_fail(main_area != NULL);
  g_return_if_fail(GTK_IS_MATH_VIEW(main_area));

  math_view = GTK_MATH_VIEW(main_area);

  gtk_math_view_set_font_size(math_view, size);
}

static void
change_link_color(GtkWidget* widget, gpointer data)
{
}

static void
select_color(const gchar* title, GtkSignalFunc callback)
{
  GtkWidget* cs = gtk_color_selection_dialog_new(title);

  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(cs)->ok_button),
		     "clicked", callback, NULL);

  gtk_signal_connect_object(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(cs)->ok_button),
			    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    (gpointer) cs);

  gtk_signal_connect_object(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(cs)->cancel_button),
			    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    (gpointer) cs);

  gtk_widget_show(cs);
}

static void
options_link_color(GtkWidget* widget, gpointer data)
{
  select_color("Select Color for Links", GTK_SIGNAL_FUNC(change_link_color));
}

static void
options_verbose(GtkWidget* widget, gpointer data)
{
  printf("not implemented %p %p\n", widget, data);
}

static void
options_select_mode(GtkWidget* widget, gpointer data)
{
  GtkMathView* math_view;
  SelectMode mode;

  g_return_if_fail(main_area != NULL);
  g_return_if_fail(GTK_IS_MATH_VIEW(main_area));

  math_view = GTK_MATH_VIEW(main_area);

  mode = gtk_math_view_get_select_mode(math_view);
  if (mode == MATH_VIEW_SELECT_PRESENTATION)
    gtk_math_view_set_select_mode(math_view, MATH_VIEW_SELECT_CONTENT);
  else
    gtk_math_view_set_select_mode(math_view, MATH_VIEW_SELECT_PRESENTATION);
}

static void
help_about(GtkWidget* widget, gpointer data)
{
  GtkWidget* dialog;
  GtkWidget* label;
  GtkWidget* ok;

  dialog = gtk_dialog_new();
  label = gtk_label_new("\n    MathML Viewer    \n    Copyright (C) 2000 Luca Padovani    \n");
  ok = gtk_button_new_with_label("Close");

  gtk_signal_connect_object (GTK_OBJECT (ok), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) dialog);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     ok);

  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);

  gtk_widget_show_all (dialog);
}

static void
create_widget_set()
{
  GtkWidget* main_vbox;
  GtkWidget* menu_bar;

  main_vbox = gtk_vbox_new(FALSE, 1);
  gtk_container_border_width(GTK_CONTAINER(main_vbox), 1);
  gtk_container_add(GTK_CONTAINER(window), main_vbox);
  gtk_widget_show(main_vbox);

  menu_bar = get_main_menu();
  gtk_box_pack_start(GTK_BOX(main_vbox), menu_bar, FALSE, TRUE, 0);
  gtk_widget_show(menu_bar);


  main_area = gtk_math_view_new(NULL, NULL);
  gtk_widget_show(main_area);

  scrolled_area = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_area),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_area);
  gtk_container_add(GTK_CONTAINER(scrolled_area), main_area);
  gtk_box_pack_end(GTK_BOX(main_vbox), scrolled_area, TRUE, TRUE, 0);

  gtk_widget_show(main_vbox);
}

GtkWidget*
get_main_menu()
{
  GtkItemFactory* item_factory;
  GtkAccelGroup* accel_group;
  gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

  accel_group = gtk_accel_group_new();

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel_group);

  gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);

  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);

  return gtk_item_factory_get_widget(item_factory, "<main>");
}
